#   -*- coding: utf-8 -*-
import os
import re
import sys
import json
from pybuilder.core import init
from pybuilder.core import task
from pybuilder.core import depends

from anybadge import Badge


@init
def init_anybadge(project):
    """ initialize anybadge task
    """
    project.plugin_depends_on('anybadge')
    project.set_property_if_unset('anybadge_exclude', [])
    project.set_property_if_unset('anybadge_add_to_readme', False)
    project.set_property_if_unset('anybadge_complexity_use_average', False)


@task('anybadge', description='generate badges from reports using anybadge')
@depends('prepare')
def anybadge(project, logger, reactor):
    """ generate badges from reports using anybadge
    """
    reports_directory = project.expand_path('$dir_reports')
    images_directory = get_images_directory(project)
    add_to_readme = project.get_property('anybadge_add_to_readme')
    exclude = get_badge_exclude(project)
    logger.debug(f'task instructed to exclude {exclude}')
    if 'complexity' not in exclude:
        report_path = f'{reports_directory}/radon'
        badge_path = f'{images_directory}/complexity.svg'
        use_average = project.get_property('anybadge_complexity_use_average')
        create_complexity_badge(report_path, badge_path, logger, add_to_readme, use_average)
    if 'severity' not in exclude:
        report_path = f'{reports_directory}/bandit.json'
        badge_path = f'{images_directory}/severity.svg'
        create_severity_badge(report_path, badge_path, logger, add_to_readme)
    if 'coverage' not in exclude:
        report_path = f'{reports_directory}/{project.name}_coverage.json'
        badge_path = f'{images_directory}/coverage.svg'
        create_coverage_badge(report_path, badge_path, logger, add_to_readme)
    if 'python' not in exclude:
        badge_path = f'{images_directory}/python.svg'
        create_python_badge(badge_path, logger, add_to_readme)


def get_images_directory(project):
    """ return images directory and create directory if does not already exist
    """
    target_directory = project.expand_path('$dir_target')
    images_directory = target_directory.replace('/target', '/docs/images')
    if not os.path.exists(images_directory):
        os.makedirs(images_directory)
    return images_directory


def get_badge_exclude(project):
    """ return list of badges to exclude
    """
    exclude = project.get_property('anybadge_exclude')
    if exclude:
        return exclude.split(',')
    return []


def accessible(filename):
    """ return True if filename is file and current context has read access otherwise False
    """
    return os.path.isfile(filename) and os.access(filename, os.R_OK)


def read_lines(filename):
    """ return content of filename as list of lines
    """
    with open(filename) as infile:
        return infile.readlines()


def read_data(filename):
    """ return dictionary read from json file
    """
    with open(filename) as infile:
        return json.load(infile)


def get_complexity_report(lines):
    """ return complexity report by analyzing lines
    """
    report = {
        'average': None,
        'highest': {
            'name': None,
            'score': 0
        }
    }
    regex_score = r'[A-Z] \d+:\d+ (?P<name>.*) - [A-Z] \((?P<score>\d+)\)'
    for line in lines[:-1]:
        line = line.strip()
        match = re.match(regex_score, line)
        if match:
            score = float(match.group('score'))
            if score > report['highest']['score']:
                report['highest']['score'] = score
                report['highest']['name'] = match.group('name')

    last_line = lines[-1].strip()
    regex_average = r'Average complexity: [A-Z] \((?P<average>.*)\)'
    match = re.match(regex_average, last_line)
    if match:
        report['average'] = float(match.group('average'))

    return report


def get_complexity_badge(complexity_report, use_average=False):
    """ return complexity badge based off of complexity_report
        https://radon.readthedocs.io/en/latest/api.html#radon.complexity.cc_rank
        1  - 5  : Simple   - Green
        6  - 10 : Stable   - Green
        11 - 20 : Slight   - Yellow
        21 - 30 : Complex  - Orange
        31 - 40 : Alarming - Red
        40+     : Unstable - BrightRed
    """
    score = complexity_report['highest']['score']
    if use_average:
        score = complexity_report['average']

    if score <= 5:
        value = 'Simple'
        color = 'green'
    elif score <= 10:
        value = 'Stable'
        color = 'green'
    elif score <= 20:
        value = 'Slight'
        color = 'yellow'
    elif score <= 30:
        value = 'Complex'
        color = 'orange'
    elif score <= 40:
        value = 'Alarming'
        color = 'red'
    else:
        value = 'Unstable'
        color = 'brightred'

    return Badge('complexity', value=value, default_color=color, num_padding_chars=1)


def get_severity_badge(severity_report):
    """ return severity badge based off of severity_report
        High      - Red
        Medium    - Orange
        Low       - Yellow
        Undefined - Gray
        None      - Green
    """
    color = 'green'
    value = 'None'

    metrics = severity_report['metrics']['_totals']
    if metrics['SEVERITY.UNDEFINED'] > 0:
        color = 'gray'
        value = 'Undefined'
    if metrics['SEVERITY.LOW'] > 0:
        color = 'yellow'
        value = 'Low'
    if metrics['SEVERITY.MEDIUM'] > 0:
        color = 'orange'
        value = 'Medium'
    if metrics['SEVERITY.HIGH'] > 0:
        color = 'red'
        value = 'High'

    return Badge('severity', value=value, default_color=color, num_padding_chars=1)


def get_coverage(coverage_data):
    """ return coverage from coveage lines
    """
    if len(coverage_data['module_names']) == 0:
        return 0
    return coverage_data['overall_coverage']


def get_coverage_badge(coverage):
    """ return coverage badge based off of coverage report
    """
    color = 'green'
    if coverage < 85:
        color = 'yellow'
    if coverage < 70:
        color = 'orange'
    if coverage < 55:
        color = 'red'
    value = f'{coverage}%'
    return Badge('coverage', value=value, default_color=color, num_padding_chars=1)


def get_python_badge():
    """ return badge for python version
    """
    value = f'{sys.version_info.major}.{sys.version_info.minor}'
    color = 'teal'
    return Badge('python', value=value, default_color=color, num_padding_chars=1)


def update_readme(name, badge_filename, add_to_readme, logger):
    """ add badge to readme
    """
    if not add_to_readme:
        return
    filename = 'README.md'
    if not accessible(filename):
        logger.warn(f'{filename} does not exist or is not accessible')
        return
    # not super clean but is the best we can do for now
    relative_filename = f"docs/images{badge_filename.split('/docs/images')[1]}"
    line_to_add = f'![{name}]({relative_filename})\n'
    with open('README.md', 'r+') as file_handler:
        lines = file_handler.readlines()
        for line in lines:
            if line.startswith(line_to_add):
                logger.debug(f'{filename} already contains {line_to_add.strip()}')
                break
        else:
            file_handler.seek(0)
            lines.insert(0, line_to_add)
            file_handler.writelines(lines)


def create_complexity_badge(report_filename, badge_filename, logger, add_to_readme, use_average):
    """ create complexity badge from radon report
    """
    if not accessible(report_filename):
        logger.warn(f'{report_filename} does not exist or is not accessible')
        return
    lines = read_lines(report_filename)
    report = get_complexity_report(lines)
    badge = get_complexity_badge(report, use_average=use_average)
    logger.info(f'writing complexity badge {badge_filename}')
    badge.write_badge(badge_filename, overwrite=True)
    update_readme('complexity', badge_filename, add_to_readme, logger)


def create_severity_badge(report_filename, badge_filename, logger, add_to_readme):
    """ create severity badge from bandit report
    """
    if not accessible(report_filename):
        logger.warn(f'{report_filename} does not exist or is not accessible')
        return
    severity_report = read_data(report_filename)
    badge = get_severity_badge(severity_report)
    logger.info(f'writing severity badge {badge_filename}')
    badge.write_badge(badge_filename, overwrite=True)
    update_readme('severity', badge_filename, add_to_readme, logger)


def create_coverage_badge(report_filename, badge_filename, logger, add_to_readme):
    """ create coverage badge from coverage report
    """
    if not accessible(report_filename):
        logger.warn(f'{report_filename} does not exist or is not accessible')
        return
    data = read_data(report_filename)
    coverage = get_coverage(data)
    badge = get_coverage_badge(coverage)
    logger.info(f'writing coverage badge {badge_filename}')
    badge.write_badge(badge_filename, overwrite=True)
    update_readme('coverage', badge_filename, add_to_readme, logger)


def create_python_badge(badge_filename, logger, add_to_readme):
    badge = get_python_badge()
    logger.info(f'writing python version badge {badge_filename}')
    badge.write_badge(badge_filename, overwrite=True)
    update_readme('python', badge_filename, add_to_readme, logger)



#!/usr/bin/env python
#   -*- coding: utf-8 -*-

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'pybuilder-anybadge',
        version = '0.2.0',
        description = 'Pybuilder plugin for anybadge badge generator',
        long_description = "[![GitHub Workflow Status](https://github.com/soda480/pybuilder-anybadge/workflows/build/badge.svg)](https://github.com/soda480/pybuilder-anybadge/actions)\n[![Code Coverage](https://codecov.io/gh/soda480/pybuilder-anybadge/branch/main/graph/badge.svg)](https://codecov.io/gh/soda480/pybuilder-anybadge)\n[![Code Grade](https://www.code-inspector.com/project/20103/status/svg)](https://frontend.code-inspector.com/project/20103/dashboard)\n[![PyPI version](https://badge.fury.io/py/pybuilder-anybadge.svg)](https://badge.fury.io/py/pybuilder-anybadge)\n\n# pybuilder-anybadge #\n\nA pybuilder plugin that generates badges for your project using `anybadge`, for more information refer to the [anybadge pypi page](https://pypi.org/project/anybadge/).\n\nTo add this plugin into your pybuilder project, add the following line near the top of your build.py:\n```python\nuse_plugin('pypi:pybuilder_anybadge')\n```\n\n**NOTE** if you are using Pybuilder version `v0.11.x`, then specify the following version of the plugin:\n```python\nuse_plugin('pypi:pybuilder_anybadge', '~=0.1.1')\n```\n\n### Pybuilder anybadge properties ###\n\nThe pybuilder task `pyb anybadge` will use anybadge to generate badges for your project by processing reports produced from various plugins; the badges that are currently supported are:\n- **complexity** - requires the [pybuilder_radon](https://pypi.org/project/pybuilder-radon/) plugin. Generate badge using cyclomatic complexity score of your most complicated function.\n- **severity** - requires the [pybuilder_bandit](https://pypi.org/project/pybuilder-bandit/) plugin. Generate badge using number of security vulnerabilities discovered by severity.\n- **coverage** - requires the `coverage` plugin. Generate badge for overall unit test coverage.\n- **python** - Generate badge for version of Python being used\n\nThe plugin will write the respective badges to the `docs/images` folder. The following plugin properties are available to further configure badge generation.\n\nName | Type | Default Value | Description\n-- | -- | -- | --\nanybadge_exclude | str | '' | Comma delimited string of badges to exclude from processing, valid values are 'complexity', 'severity', 'coverage' and 'python'\nanybadge_add_to_readme | bool | False | Specify if plugin should add badges to the README.md file (see below for example). **Note** the plugin will add the badge references but you must commit/push the changes (including svg files in the docs/images folder)\nanybadge_complexity_use_average | bool | False | Use overall average complexity as score when generating complexity badge\n\nThe plugin properties are set using `project.set_property`, the following is an example of how to set the properties:\n\n```Python\nproject.set_property('anybadge_exclude', 'severity,coverage')\nproject.set_property('anybadge_add_to_readme', True)\nproject.set_property('anybadge_complexity_use_average', True)\n```\n\nThe following badges were generated for this project using the `pybuilder_anybadge` plugin:\n\n![coverage](https://raw.githubusercontent.com/soda480/pybuilder-anybadge/main/docs/images/coverage.svg)\n![severity](https://raw.githubusercontent.com/soda480/pybuilder-anybadge/main/docs/images/severity.svg)\n![complexity](https://raw.githubusercontent.com/soda480/pybuilder-anybadge/main/docs/images/complexity.svg)\n![python](https://raw.githubusercontent.com/soda480/pybuilder-anybadge/main/docs/images/python.svg)\n\n### Development ###\n\nClone the repository and ensure the latest version of Docker is installed on your development server.\n\nBuild the Docker image:\n```sh\ndocker image build \\\n-t \\\npybanybadge:latest .\n```\n\nRun the Docker container:\n```sh\ndocker container run \\\n--rm \\\n-it \\\n-v $PWD:/pybuilder-anybadge \\\npybanybadge:latest \\\n/bin/sh\n```\n\nExecute the build:\n```sh\npyb -X\n```",
        long_description_content_type = 'text/markdown',
        classifiers = [
            'Development Status :: 4 - Beta',
            'Environment :: Other Environment',
            'Environment :: Plugins',
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: Apache Software License',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3.6',
            'Topic :: Software Development :: Build Tools'
        ],
        keywords = '',

        author = 'Emilio Reyes',
        author_email = 'soda480@gmail.com',
        maintainer = '',
        maintainer_email = '',

        license = 'Apache License, Version 2.0',

        url = 'https://github.com/soda480/pybuilder-anybadge',
        project_urls = {},

        scripts = [],
        packages = ['pybuilder_anybadge'],
        namespace_packages = [],
        py_modules = [],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = ['anybadge'],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        python_requires = '',
        obsoletes = [],
    )
